#if !defined(IR_MANAGER_HH)
#define IR_MANAGER_HH

#include <map>
#include <vector>

#include "AstManager.hh"

///////////////////////////////////////////////////////////////////////////
class IrManager : public AstManager { 
///////////////////////////////////////////////////////////////////////////

public :
  // Special variable identifiers
  enum IdTypes {
    ERR_ID = -1,
    NIL_ID = 0
  };

  // Constructor and destructor
  IrManager( void ) { 
    id2string_.push_back("");
  }
  ~IrManager( void ) { }

  // Overriding inherited methods
  // [TO DO: Why do I have to add these?]
  AstNode* newOperator() { return AstManager::newOperator(); }
  AstNode* newOperand() { return AstManager::newOperand(); }

  // Additional factor methods
  AstNode* newOperator( AstNode::OpType ot ) { return (new Operator(ot)); }
  AstNode* newOperand( int value ) { return (new Operand(value)); }
  AstNode* newVariable( int value ) { return (new Variable(value)); }

  // The find or add method to hash identifiers and give them unique IDs
  int findOrAddId(const string& identifier) {
    int id = NIL_ID;
    std::map<string,int>::iterator i = string2id_.find(identifier);
    if (i != string2id_.end()) {
      // The identifier has already been declared
      id = i->second;
    } else {
      // The identifier has not been already declared 
      // A fresh unique id must be assigned to it and returned
      string2id_[identifier] = id = id2string_.size();
      id2string_.push_back(identifier);
    }
    return id;
  }
  // The find method to get unique ids for hashed identifiers
  // NIL_ID is returned when search fails
  int findId(const string& identifier) {
    int id = NIL_ID;
    std::map<string,int>::iterator i = string2id_.find(identifier);
    if (i != string2id_.end()) {
      // The identifier has already been declared
      id = i->second;
    }
    return id;
  }
  // The find name to get the unique id given the identifier
  const string& findName(int id) {
    assert(id < id2string_.size());
    return id2string_[id];
  }

  // Redefining the print method
  virtual void print( std::ostream& ofs, AstNode* n ) {
    assert(n != 0);
    // Print the current statement
    if (n->isOperand()) {
      if (n->isVariable()) {
	ofs << findName(n->getValue());
      } else {
	ofs << n->getValue();
      }
    } else { // n->isOperator()
      if (n->getOpType()==AstNode::ISEQ) {
	print(ofs, n->getOperand(AstNode::LEFT));
	ofs << " = ";
	print(ofs, n->getOperand(AstNode::RIGHT));
      } else {
	ofs << "(";
	print(ofs, n->getOperand(AstNode::LEFT));
	switch (n->getOpType()) {
	case AstNode::PLUS : ofs << " + "; break;
	case AstNode::MINUS : ofs << " - "; break;
	case AstNode::TIMES : ofs << " * "; break;
	case AstNode::FRAC : ofs << " / "; break;
	default : ofs << " ? "; break;
	}
	print(ofs,n->getOperand(AstNode::RIGHT));
	ofs << ")";
      }
    }
    // Print the next statement(s)
    if (n->getNext() != 0) {
      ofs << std::endl;
      print(ofs,n->getNext());
    } 
  }

private :
  std::map<string, int> string2id_;
  std::vector<string>   id2string_;

};
#endif
